@@ -532,7 +532,7 @@ def vif(X, sorted=True) -> pd.DataFrame:
 
 def condition_index(df) -> pd.DataFrame:
     """
-    Calculates the Condition Index for each feature in a DataFrame to assess multicollinearity.
+    Calculates the Condition Index for a DataFrame to assess multicollinearity.
 
     The Condition Index is computed based on the eigenvalues of the covariance matrix
     of the standardized data. High condition indices suggest potential multicollinearity issues.
@@ -542,9 +542,9 @@ def condition_index(df) -> pd.DataFrame:
 
     Returns:
         pandas.DataFrame: A DataFrame with the following columns:
-            - 'Feature': The name of the feature.
+            - 'Index': The index of the eigenvalue.
             - 'Eigenvalue': The eigenvalue of the covariance matrix.
-            - 'Condition Index': The Condition Index for the feature.
+            - 'Condition Index': The Condition Index for the eigenvalue.
 
     Examples:
         >>> from spotpython.utils.stats import condition_index
@@ -555,28 +555,27 @@ def condition_index(df) -> pd.DataFrame:
         ...     'x3': [1, 3, 5, 7, 9]
         ... })
         >>> condition_index(data)
-           Feature  Eigenvalue  Condition Index
-        0      x1    1.140000         1.000000
-        1      x2    0.000000              inf
-        2      x3    0.002857        20.000000
+           Index  Eigenvalue  Condition Index
+        0      0    1.140000         1.000000
+        1      1    0.000000              inf
+        2      2    0.002857        20.000000
     """
-    # Standardisieren der Daten
+    # Standardize the data
     X = df.values
     X_centered = X - np.mean(X, axis=0)
 
-    # Berechnung der Kovarianzmatrix
+    # Compute the covariance matrix
     covariance_matrix = np.cov(X_centered, rowvar=False)
 
-    # Berechnung der Eigenwerte der Kovarianzmatrix
+    # Compute the eigenvalues of the covariance matrix
     eigenvalues, _ = np.linalg.eigh(covariance_matrix)
 
-    # Berechnung des Condition Index
-    # Condition Index ist die Wurzel des Verhältnisses des größten Eigenwertes zum jeweiligen Eigenwert
+    # Handle division by zero for eigenvalues
     max_eigenvalue = max(eigenvalues)
-    condition_indices = np.sqrt(max_eigenvalue / eigenvalues)
+    condition_indices = np.array([np.sqrt(max_eigenvalue / ev) if ev > 0 else np.inf for ev in eigenvalues])
 
-    # Erstellen eines DataFrames zur Anzeige der Ergebnisse
-    condition_index_df = pd.DataFrame({"Feature": df.columns, "Eigenvalue": eigenvalues, "Condition Index": condition_indices})
+    # Create a DataFrame for the results
+    condition_index_df = pd.DataFrame({"Index": range(len(eigenvalues)), "Eigenvalue": eigenvalues, "Condition Index": condition_indices})
 
     return condition_index_df
 
@@ -786,24 +785,36 @@ def preprocess_df_for_ols(df, independent_var_columns, target_col) -> tuple:
     return X_encoded, y
 
 
-def get_combinations(z_ind: list) -> list:
+def get_combinations(ind_list: list, type="indices") -> list:
     """
     Generates all possible combinations of two targets from a list of target indices. Order is not important.
 
     Args:
-        z_ind (list): A list of target indices.
+        ind_list (list): A list of target indices.
 
     Returns:
         list: A list of tuples, where each tuple contains a combination of two target indices.
              The order of the targets within a tuple is not important, and each combination
              appears only once.
+        type (str): The type of output, either 'values' or 'indices'. Default is 'indices'.
 
     Examples:
         >>> from spotpython.utils.stats import get_combinations
-        >>> z_ind = [0, 1, 2, 30]
-        >>> combinations = get_combinations(z_ind)
-        >>> print(combinations)
-        [(0, 1), (0, 2), (0, 30), (1, 2), (1, 30), (2, 30)]
+        >>> ind_list = [0, 10, 20, 30]
+        >>> combinations = get_combinations(ind_list)
+        >>> combinations = get_combinations(ind_list, type='indices')
+            [(0, 1), (0, 2), (0, 3), (1, 2), (1, 3), (2, 3)]
+        >>> print(combinations, type='values')
+            [(0, 10), (0, 20), (0, 30), (1, 20), (1, 30), (2, 30)]
     """
-    combinations = [(z_ind[i], z_ind[j]) for i in range(len(z_ind)) for j in range(i + 1, len(z_ind))]
+    # check that ind_list is a list
+    if not isinstance(ind_list, list):
+        raise ValueError("ind_list must be a list.")
+    m = len(ind_list)
+    if type == "values":
+        combinations = [(ind_list[i], ind_list[j]) for i in range(m) for j in range(i + 1, m)]
+    elif type == "indices":
+        combinations = [(i, j) for i in range(m) for j in range(i + 1, m)]
+    else:
+        raise ValueError("type must be either 'values' or 'indices'.")
     return combinations
